package dnibuilder

import java.io.FileReader
import org.eclipse.xtend.lib.annotations.Data
import static extension com.google.common.io.CharStreams.*
import java.util.List
import java.io.File
import java.util.ArrayList

@Data class Generator {
	String name
	int scale
	int value
	double begin
	double end
	
	override toString(){
		return "GEN(B="+begin+", S="+scale+", V="+value+", E="+end+")"
	}
}

@Data class IPPair {
	String name
	List<Generator> generators 
}

@Data class CaseStudy {
	String orgPcapngFileNamePrefix
	List<String> orgPcapngFileNames
	List<IPPair> ippairs 
}


class DirectoryParser{
	
//	List<String> csvOutFiles 
	
	def parse(String path){
		val dir = new File(path)
		if(dir.directory){
			return dir.listFiles[p | dir.directory && p.name.endsWith(".csv.list")].map[f | f.name]
		}
		return null
	}
}



// Discover separate case studies for set of files
// assign files for a case study
// create empty flows and assign a file representing the flow
class CaseStudies{
	
	List<CaseStudy> caseStudyObjects = new ArrayList<CaseStudy>()
	
	def getCaseStudies(){
		caseStudyObjects
	}
	
	def read(List<String> list){
		val caseStudyNames = list.map[fn|fn.split('_data').get(0)].toSet
		caseStudyObjects += caseStudyNames.map[csn | new CaseStudy( csn, list.filter[fn | fn.startsWith(csn)].toList, new ArrayList<IPPair>())];
		
		extractFlows();
	}
	
	def extractFlows(){
		for(cso: caseStudyObjects){
			for(pcapngName: cso.orgPcapngFileNames){
				cso.ippairs += new IPPair(pcapngName, new ArrayList<Generator>())	
			}
		}
	}
	
	def extractGenerators(String directory){
		for(cso: caseStudyObjects){
			for(ippair: cso.ippairs){
				val path = directory+"/"+ippair.name
				ippair.generators.addAll(getGeneratorsFromFile(path))
			}
			
		}
	}
	
	def getGeneratorsFromFile(String fname){
		new FileReader(fname).readLines.map[ line | extractGeneratorFromLine(line) ].filter[g|g!=null]
	}
	
	def extractGeneratorFromLine(String line){
		if(line.startsWith('scale')){
			return null
		}
		else if(!line.isEmpty){
//			println("non-empty "+line)
			
			val segments = line.split(',').iterator
			return new Generator(
				"genname", 
				Integer.parseInt(segments.next), //scale
				Integer.parseInt(segments.next), //value
				Double.parseDouble(segments.next), //begin
				Double.parseDouble(segments.next) //end
			)
		}
		return null
	}
	


}

