package epsilonStandalone

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.ParseConversionEvent;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import java.util.LinkedHashMap
import java.util.LinkedList

class SimQpnResultsParser {

	public var files  = newLinkedList()
	
	public var interestingPlaces = <String> newLinkedList()
	public var interestingColors = <String> newLinkedList()
	public var interestingMetrics = <String> newLinkedList()
			
	public def String processFile(String fileName){
		
		var HashMap<String, HashMap<String, LinkedHashMap<String, Double>>> map = parse(fileName);
		
		println("WholeMap: "+map)
		
		map.forEach[placeName, colorMetricValueHM, placeIndex|
			
			println("CMV ("+placeName+"): "+colorMetricValueHM)
			
			colorMetricValueHM.forEach[colorName, metricValueHM |
				metricValueHM.forEach[metricName, value |
					
					println("place: "+placeName+" colorName "+colorName+" metricName "+metricName+" values: "+value)
					var double tokensPerSec = value
					var double bytes = parseColorMessageSize(colorName, 1024);	
					var double bytesPerSec = tokensPerSec * bytes;
				]
			]
		]
		
		
		return map.toString();
	}
			

	
	public def void getSimqpnFiles(String directory){
		
		var File folder = new File(directory);
		System.out.println("Reading dir " + directory);
		
		var listOfFiles = folder.listFiles();
		for (var  i = 0; i < listOfFiles.length; i++) {
			
			if (listOfFiles.get(i).isFile()) {
				var String fname = listOfFiles.get(i).getName(); 
				//System.out.println("File " + fname);
				if(fname.endsWith(".simqpn") && fname.startsWith("SimQPN_Output")){
					var String path = directory+"/"+fname; 
					System.out.println("Filepath " + path);
					files.add(path);
				}
			}
		}
	}
	public def double parseColorMessageSize(String color, int kiloMultipler ){
		
//		System.out.println("color to parse: "+color);

		//Flow-iperf_to_iperf-destination_size_1G_145
		
		var String[] tokens = color.split("_");
		
		//var long kiloMultipler = 1024

		var boolean size_found = false;
		for (String t : tokens){
			if(size_found){
				var String token = t
				
				var long multipler = 1;
				if(t.contains("G")){
					multipler = kiloMultipler*kiloMultipler*kiloMultipler
					token = t.replace("G","")
					
				}
				else if (t.contains("M")){
					multipler = kiloMultipler*kiloMultipler
					token = t.replace("M","")
				}
				else if(t.contains("K")){
					multipler = kiloMultipler
					token = t.replace("K","")
				}
				else{
					multipler = 1
				}
				//println("token: "+token+" mult "+multipler)
				try {
					var double l = Double.valueOf(token).doubleValue();

					return multipler * l;
				} catch (NumberFormatException nfe) {
					
				}
				size_found = false
			}
			if(t.equalsIgnoreCase("size")){
				size_found = true;
			}
		}
		return 0;
	}
	

	
	
//	public def String hashMapToString(HashMap<String, List<Double>> hm){
//		var DecimalFormat f = new DecimalFormat("000");
//		var String str = "";
//		
//		var Set set = hm.entrySet();
//		var Iterator i = set.iterator();
//	    while(i.hasNext()) {
//	         var Map.Entry me = i.next() as Map.Entry;
//	         
//	         var List<Double> l =  me.getValue() as List<Double>;
//	         var String name = me.getKey() as String;
//	         
//	         if(isInterestingPlace(name)){
//	        	 str += f.format(l.get(0));
//	        	 str += " ";
//	        	 str += f.format(l.get(1));
//	        	 str += " ";
//	        	 str += name; //getFriendlyName(name);
//	        	 str += System.getProperty("line.separator");
//	         }
//	    }
//	    return str;
//	}
	
	public def boolean isInterestingColor(String name){
		
		val x = interestingColors.filter[pn | name.contains(pn)].size()>0
		return x
		
	}
	
	public def boolean isInterestingPlace(String name){
		val x = (interestingPlaces.filter[pn | name.contains(pn)].size()>0)
		return x
	}
	public def boolean isInterestingMetric(String name){
		val x = (interestingMetrics.filter[pn | name.equals(pn)].size()>0)
		return x
	}	
	public def String getFriendlyName(String name){
	
		var list = interestingPlaces.filter[pn | name.contains(pn)]
		if(list.empty){
			return name
		}
		else{
			return list.head
		}
	}
	
	
	public def HashMap<String, HashMap<String, LinkedHashMap<String, Double>>> parse(String fname) throws ParserConfigurationException, SAXException, IOException{
		
		
		
		
		//place -> colors[] -> metrics[] -> value
		var HashMap<String, HashMap<String, LinkedHashMap<String, Double>>> placeColorsMetricValueHashMap = new HashMap<String, HashMap<String, LinkedHashMap<String, Double>>>();
		
		
		var File fXmlFile = new File(fname);
		var DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
		var DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
		var Document doc = dBuilder.parse(fXmlFile);
		doc.getDocumentElement().normalize();
		
		var NodeList nList = doc.getElementsByTagName("observed-element");

		for (var int i = 0; i < nList.getLength(); i++) {
		
			  //We have encountered an <employee> tag.
			var Node node = nList.item(i);
			  
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				
				var Element obsElement = node as Element;
				
				var String obsElementName = obsElement.getAttribute("name") as String
				var String obsElementType = obsElement.getAttribute("type") as String
				
			
			    
			    if( (isInterestingPlace(obsElementName) || interestingPlaces.isEmpty())){
			    	
			    	var HashMap<String, LinkedHashMap<String, Double>> colorMetricsHashMap = new HashMap<String, LinkedHashMap<String, Double>>();
			    	//placeColorsMetricHashMap.put(obsElementName, colorMetricHashMap); //put is done at the end of the loop
			    	
			    	

		    		var obsElementColors = obsElement.getElementsByTagName("color")
		    		
			    	for (var int j = 0; j < obsElementColors.getLength(); j++) {
				    	var Node colorNode = obsElementColors.item(j);
				    	
				    	
				    	if (colorNode != null && colorNode instanceof Element) {
				    		
				    		var String colorName = colorNode.getAttributes().getNamedItem("name").getNodeValue();
				    		
				    		if(isInterestingColor(colorName) || interestingColors.isEmpty()){
				    			var metrics = newLinkedHashMap //LinkedHashMap<String, Double>
					    		//colorMetricHashMap.put(colorName, metricsValues) //put is done at the end of the loop
					    		
					    		var colorMetrics = (colorNode as Element).getElementsByTagName("metric")
					    		
	
					    		for (var int k = 0; k < colorMetrics.getLength(); k++) {
							    	var Node metricNode = colorMetrics.item(k);
							    	
							    	if (metricNode instanceof Element) {
							    		var String metricType = metricNode.getAttributes().getNamedItem("type").getNodeValue();
							    		var String metricValue = metricNode.getAttributes().getNamedItem("value").getNodeValue();
							    		if(isInterestingMetric(metricType) || interestingMetrics.empty){
							    			var d_value = Double.parseDouble(metricValue);
							    			metrics.put(metricType, d_value)
							    		}
							    	}
					    		}
					    		colorMetricsHashMap.put(colorName, metrics)
				    		
				    		}//isInterestingColor
				    		
				    				
				      		
				    	}//if color defined
			    	}//for colors

			    	placeColorsMetricValueHashMap.put(obsElementName, colorMetricsHashMap);
			    }
			 }
		}
		return placeColorsMetricValueHashMap;
	}
}